/*  Copyright Ton Dieker                                                    */
/*  Centre of Mathematics and Computer Science (CWI) Amsterdam              */
/*  April 2002                                                              */

/*  ton@cwi.nl                                                              */


void 	FGN_spectrum ( double *pow_spec, int size, double H);
double 	FGN_B_est (double lambda, double H);
double 	FGN_B_est_adj (double lambda, double H);


